/*
* $AvrDtmf.c:Danny H. Newport:3-25-03
*
* Danny H. Newport
* compiled with AvrEdit Ver. 3.5 using 29% memory (room for expansion)
*
*	Main Parts
*		AVR 2313 @ 4mhz
*		8870 dtmf decoder
*
*	Port layout
* 		PORTB	Pin0	Q1
*				Pin1	Q2
*				Pin2	Q3
*				Pin3	Q4
*				Pin4	StD	goes high when valid dtmf signal is heard
*		PORTD
*				PD0		TX	used for displaying valid dtmf tones
*				PD1		RX
*				
*
*   This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   
*
*
*	Special Thanks.
*		AVR GCC team for a great compiler
*		AvrEdit for a great avr ide setup
*		Atmel for the versitile Micro.    
* 
*/

#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#include <timer.h>
#define __AVR_AT90S2313__


//defines for delay
#define CPU_freq    4000000						//cpu clock frequency
#define CYCLES_PER_US ((CPU_freq+500000)/1000000) 	// cpu cycles per microsecond
#undef ECHO
#define UART_BAUD_RATE 9600
#define UART_BAUD_SELECT ((CPU_freq/(UART_BAUD_RATE*16))-1)

typedef unsigned char  u08;
/* global variables used by PWM outputs */
volatile uint8_t pwm[8];
volatile uint8_t pwm_config;

/* uart globals */
/*static volatile u08 *uart_data_ptr;
static volatile u08 uart_counter;*/

/* global variables for delay functions */
uint16_t delay_count=CPU_freq/4000;				// used in delay function

/* prototypes */
//void dobuffer (void);

/* global variables for serial port */
//static uint8_t recv_bytes = 0;
//static char buffer[10];
volatile uint8_t ready = 0;
uint8_t m_dtmf;
uint8_t m_time=0;
uint8_t m_found=0;


/*
* Set default values
*/
void init(void)
{
	m_dtmf = 0;
	
}


/*
* uart transmit complete isr
*/
INTERRUPT (SIG_UART_TRANS)
{
	ready = 1;
}


// millisecond delay function
void delay_ms(uint8_t ms)
{
	uint16_t cnt; asm volatile (
		
		"\n" "L_dl1%=:" "\n\t" "mov %A0, %A2" "\n\t" "mov %B0, %B2" "\n" "L_dl2%=:" "\n\t"
		
		"sbiw %A0, 1" "\n\t" "brne L_dl2%=" "\n\t" "dec %1" "\n\t" "brne L_dl1%=" "\n\t" : "=&w" (cnt) : "r" (ms), "r" (delay_count) );
}

/*
*  function for endless loop
*/
void main_loop (void)
{
	
	//display Ready on terminal upon powerup
	outp ('R', UDR);
    delay_ms(1);
	outp ('e',UDR);
    delay_ms(1);
	outp ('a', UDR);
    delay_ms(1);
	outp ('d',UDR);
    delay_ms(1);
	outp ('y', UDR);
    delay_ms(1);
	outp ('\n',UDR);
	
	delay_ms(100);
	
	for (;;)
	{
		
               
		while(bit_is_set(PINB,PINB4)==0)	//wait for a valid tone from the 8870
		{
			//FOUND VALID DTMF TONE
			
			m_time+=1;
			delay_ms(50);
			
			//add a newline after 15 seconds if no new tone heard
			if(m_time>200&&m_found==1)
			{
				m_time=0;
				m_found=0;
				outp ('\n', UDR);
				delay_ms(10);
				
			}
			else if(m_time==40&&m_found==1)//add a space if no tone for 2 seconds
				outp(' ',UDR);
			
		}//end while
		
		m_dtmf=0;
		m_found=1;
		m_time=0;
		//poor way of finding first 4 bits....update later.
		if(bit_is_clear(PINB,PINB0)==0)
			m_dtmf+=1;
		
		if(bit_is_clear(PINB,PINB1)==0)
			m_dtmf+=2;
		
		if(bit_is_clear(PINB,PINB2)==0)
			m_dtmf+=4;
		
		if(bit_is_clear(PINB,PINB3)==0)
			m_dtmf+=8;
		
		switch(m_dtmf)
		{
		case 0:
			outp('D',UDR);
			break;
		case 1:
			outp('1',UDR);
			break;
		case 2:
			outp('2',UDR);
			break;
		case 3:
			outp('3',UDR);
			break;
		case 4:
			outp('4',UDR);
			break;
		case 5:
			outp('5',UDR);
			break;
		case 6:
			outp('6',UDR);
			break;
		case 7:
			outp('7',UDR);
			break;
		case 8:
			outp('8',UDR);
			break;
		case 9:
			outp('9',UDR);
			break;
		case 10:
			outp('0',UDR);
			break;
		case 11:
			outp('*',UDR);
			break;
		case 12:
			outp('#',UDR);
			break;
		case 13:
			outp('A',UDR);
			break;
		case 14:
			outp('B',UDR);
			break;
		case 15:
			outp('C',UDR);
			break;
		default:
			outp('?',UDR);
			break;
		}
     
        

	
	delay_ms(2);
	//wait until tone has cleared before going on.
        loop_until_bit_is_clear(PINB,PINB4);
        /*while(bit_is_set(PINB,PINB4)==0)
        {
        outp('+',UDR);
        }*/
	}
}



/*
*
*/
int
main (void)
{
	init();
	
	outp (0x00, DDRB);			/* portb = input */
	outp (0x00, DDRD);			/* portd = input//not really used yet */
	
	outp ((1 << TXCIE) | (1 << RXEN) | (1 << TXEN), UCR);
	//outp (64, UBRR);//use this for 10mhz
	outp (25, UBRR);//use this for 4mhz
	sei ();						/* enable interrupts */
	main_loop ();
	return 0;
}



/*void delay_ms(char delay_time)
{
int c;
for(c = 0; c < (CPU_freq / 40000); c++)
{
}
}*/

/*// microsecond delay function
void delay_us(unsigned short us)
{
unsigned short  delay_loops;
register unsigned short  i;

  delay_loops = (us+3)/5*CYCLES_PER_US; // +3 for rounding up (dirty)
  
	// one loop takes 5 cpu cycles
	for (i=0; i < delay_loops; i++) {};
}*/
